// Generated by CoffeeScript 2.2.3
(function() {
  var $, Chalk, ChildProcess, Fs, Groups, Health, Nginx, Output, SlimProcess, Tnet, _restart_action, addAction, addGroup, configFile, echo, echoResponse, exists, expandPath, formatUptime, healthSymbol, int, nginxTemplate, removeAction, removeGroup, required, saveConfig, simpleAction, statusAction, trueFalse, verbose, warn, yesNo;

  ({$, echo, warn, verbose} = require('./common'));

  Fs = require('fs');

  Tnet = require('./util/tnet');

  Chalk = require('chalk');

  Nginx = require('./daemon/nginx');

  Health = require('./daemon/health');

  Output = require('./daemon/output');

  SlimProcess = require('./util/process-slim');

  ChildProcess = require('child_process');

  int = function(n) {
    return parseInt(n != null ? n : 0, 10);
  };

  ({yesNo, formatUptime, trueFalse} = require("./format"));

  ({configFile, exists, nginxTemplate, expandPath} = require("./files"));

  saveConfig = null;

  healthSymbol = function(v) {
    switch (v) {
      case void 0:
        return Chalk.yellow("?");
      case true:
        return Chalk.green("\u2713");
      case false:
        return Chalk.red("x");
    }
  };

  ({Groups, addGroup, removeGroup, simpleAction} = require("./daemon/groups"));

  required = function(msg, key, label) {
    if (!msg) {
      return warn("msg is required.");
    }
    if (!(msg[key] && msg[key].length)) {
      return warn(`${label} is required.`);
    }
    return true;
  };

  echoResponse = function(resp, socket) {
    console.log("Daemon says:", resp);
    return socket.end();
  };

  Object.assign(module.exports, {
    Actions: {
      // Adding a group
      add: addAction = {
        options: [["--group <group>", "Name of the group to create."], ["--cd <path>", "The working directory to spawn processes in."], ["--exec <script>", "Any shell command, e.g. 'node app.js'."], ["--count <n>", "The starting size of the group.", int], ["--port <port>", "If specified, set PORT in env for each child, incrementing port each time.", int], ["--grace <ms>", "How long to wait for a process to start."]],
        toMessage: function(cmd) {
          return {
            c: 'add',
            g: cmd.group,
            d: cmd.cd,
            x: cmd.exec,
            n: cmd.count,
            p: cmd.port,
            ms: cmd.grace
          };
        },
        onMessage: function(msg, client, cb) {
          var acted;
          acted = required(msg, 'g', "--group is required with 'add'") && required(msg, 'x', "--exec is required with 'add'") && addGroup(msg.g, msg.d, msg.x, msg.n, msg.p, msg.ms, cb);
          if (client != null) {
            client.write($.TNET.stringify((acted ? `Group ${msg.g} added.` : "No group added.")));
          }
          return typeof cb === "function" ? cb(null, acted) : void 0;
        },
        onResponse: echoResponse
      },
      // Removing a group
      remove: removeAction = {
        options: [["--group <group>", "Name of the group to create."]],
        toMessage: function(cmd) {
          return {
            c: 'remove',
            g: cmd.group
          };
        },
        onMessage: function(msg, client, cb) {
          var acted;
          acted = required(msg, 'g', "--group is required with 'add'") && removeGroup(msg.g);
          if (client != null) {
            client.write($.TNET.stringify((acted ? `Group ${msg.g} removed.` : "No group removed.")));
          }
          return typeof cb === "function" ? cb(null, acted) : void 0;
        },
        onResponse: echoResponse
      },
      // Replace (remove then add) a group
      replace: {
        options: addAction.options,
        toMessage: function(cmd) {
          return Object.assign(addAction.toMessage(cmd), {
            c: 'replace'
          });
        },
        onMessage: function(msg, client, cb) {
          echo("in replace.onMessage", msg, typeof client, typeof cb);
          return removeAction.onMessage({
            g: msg.g
          }, null, () => {
            echo("before addAction.onMessage", msg, typeof client, typeof cb);
            return addAction.onMessage(msg, client, cb);
          });
        },
        onResponse: echoResponse
      },
      // Start a group or instance.
      start: {
        options: [["--instance <id>", "Start one particular instance."], ["--group <group>", "Start all instances in a group."]],
        toMessage: function(cmd) {
          return {
            c: 'start',
            g: cmd.group,
            i: cmd.instance
          };
        },
        onMessage: simpleAction('start'),
        onResponse: echoResponse
      },
      // Stop a group or instance.
      stop: {
        options: [["--instance <id>", "Stop one particular instance."], ["--group <group>", "Stop all instances in a group."]],
        toMessage: function(cmd) {
          return {
            c: 'stop',
            g: cmd.group,
            i: cmd.instance
          };
        },
        onMessage: simpleAction('stop'),
        onResponse: echoResponse
      },
      // Restart a group or instance.
      restart: _restart_action = {
        options: [["--instance <id>", "Restart one particular instance."], ["--group <group>", "Restart all instances in a group."]],
        toMessage: function(cmd) {
          return {
            c: 'restart',
            g: cmd.group,
            i: cmd.instance
          };
        },
        onMessage: simpleAction('restart'),
        onResponse: echoResponse
      },
      reload: _restart_action, // alias
      
      // Get the status of everything.
      status: statusAction = {
        toMessage: function(cmd) {
          return {
            c: 'status'
          };
        },
        onResponse: function(resp, socket) {
          var g, group, has_pid, i, j, k, len, len1, line, pad_columns, ref, ref1;
          try {
            if (socket == null) {
              return console.log("Socket: null.");
            }
            if (!((resp != null) && ('object' === typeof resp) && ('groups' in resp))) {
              return console.log("Response:", resp);
            }
            console.log(`Status: online, pid: ${resp.pid} net: (${socket._connectLatency}ms, ${resp.send - resp.start}ms, ${Date.now() - resp.send}ms)`);
            console.log(`Groups: ${resp.groups.length}`);
            pad_columns = function(a, w = [19, 7, 7, 10, 8, 8, 14, 7, 7]) {
              var i, item;
              return ((function() {
                var j, len, results;
                results = [];
                for (i = j = 0, len = a.length; j < len; i = ++j) {
                  item = a[i];
                  results.push($.padLeft(String(item != null ? item : ''), w[i]));
                }
                return results;
              })()).join('');
            };
            ref = resp.groups;
            for (g = j = 0, len = ref.length; j < len; g = ++j) {
              group = ref[g];
              if (g === 0) {
                console.log(pad_columns(["Instance", "PID", "Port", "Uptime", "Healthy", "Enabled", "Status", "CPU", "RAM"]));
              } else {
                console.log(pad_columns(["--------", "---", "----", "------", "-------", "-------", "------", "---", "---"]));
              }
              ref1 = group.procs;
              for (i = k = 0, len1 = ref1.length; k < len1; i = ++k) {
                line = ref1[i];
                has_pid = line[1] != null;
                if (line[1] == null) {
                  line[1] = Chalk.red("-");
                }
                if (line[2] == null) {
                  line[2] = Chalk.red("-");
                }
                line[3] = formatUptime(line[3]);
                line[4] = has_pid && (healthSymbol(line[4])) || line[1];
                line[5] = healthSymbol(line[5]);
                line[7] = has_pid && (parseFloat(line[7]).toFixed(1) + "%") || line[1];
                line[8] = has_pid && ($.commaize(Math.round(line[8] / 1024)) + "mb") || line[1];
                console.log(pad_columns(line));
              }
            }
            return socket.end();
          } finally {
            if (socket != null) {
              socket.end();
            }
          }
        },
        onMessage: function(msg, client, cb) {
          var output;
          output = {
            start: Date.now(),
            pid: process.pid,
            groups: []
          };
          SlimProcess.getProcessTable((err, procs) => { // force the cache to be fresh
            if (err) {
              return typeof cb === "function" ? cb(err, false) : void 0;
            }
            Groups.forEach(function(group) {
              var _group, j, len, pcpu, pid, proc, ref, ref1, results, rss;
              output.groups.push(_group = {
                name: group.name,
                cd: group.cd,
                exec: group.exec,
                n: group.n,
                port: group.port,
                grace: group.grace,
                procs: []
              });
              results = [];
              for (j = 0, len = group.length; j < len; j++) {
                proc = group[j];
                pid = (ref = proc.proc) != null ? ref.pid : void 0;
                pcpu = rss = 0;
                if (pid != null) {
                  SlimProcess.visitProcessTree(pid, function(_p) {
                    pcpu += _p.pcpu;
                    return rss += _p.rss;
                  });
                }
                results.push(_group.procs.push([proc.id, (ref1 = proc.proc) != null ? ref1.pid : void 0, proc.port, proc.uptime, proc.healthy, proc.enabled, proc.statusString, pcpu, rss]));
              }
              return results;
            });
            output.send = Date.now();
            client.write($.TNET.stringify(output));
            return typeof cb === "function" ? cb(null, true) : void 0;
          });
          return false;
        }
      },
      stat: statusAction,
      // Disable a group or instance.
      disable: {
        options: [["--instance <id>", "Disable (stop and don't restart) one instance."], ["--group <group>", "Disable all instances in a group."]],
        toMessage: function(cmd) {
          return {
            c: 'disable',
            g: cmd.group,
            i: cmd.instance
          };
        },
        onMessage: simpleAction('disable'),
        onResponse: echoResponse
      },
      // Enable a group or instance.
      enable: {
        options: [["--instance <id>", "Enable (stop and don't restart) one instance."], ["--group <group>", "Enable all instances in a group."]],
        toMessage: function(cmd) {
          return {
            c: 'enable',
            g: cmd.group,
            i: cmd.instance
          };
        },
        onMessage: simpleAction('enable'),
        onResponse: echoResponse
      },
      // Scale the number of instance in a group up/down.
      scale: {
        options: [["--group <group>", "Which group to scale."], ["--count <n>", "How many processes should be running.", int]],
        toMessage: function(cmd) {
          return {
            c: 'scale',
            g: cmd.group,
            n: cmd.count
          };
        },
        onMessage: function(msg, client) {
          var acted;
          acted = required(msg, 'g', "--group is required with 'scale'" && (Groups.has(msg.g) || warn(`Unknown group name passed to --group ('${msg.g}')`)) && Groups.get(msg.g).scale(msg.n));
          if (acted) {
            if (client != null) {
              client.write($.TNET.stringify(`Scaled group to ${msg.n} processes.`));
            }
          } else {
            if (client != null) {
              client.write($.TNET.stringify("Nothing to scale."));
            }
          }
          return acted;
        },
        onResponse: echoResponse
      },
      // Add/remove log output destinations.
      log: {
        options: [["--list", "List the current output file."], ["--file <file>", "Send output to this log file."], ["--disable", "Stop logging to file."], ["--clear", "Clear the log file."], ["--purge", "(alias for clear)"], ["--tail", "Pipe the log output to your console now."]],
        toMessage: function(cmd) {
          var ref;
          return {
            c: 'log',
            l: trueFalse(cmd.list),
            d: cmd.disable,
            f: cmd.file,
            t: trueFalse(cmd.tail),
            p: (ref = cmd.clear) != null ? ref : cmd.purge
          };
        },
        onMessage: function(msg, client, cb) {
          var detach, handler, outputFile, ret, send;
          ret = false;
          send = (obj) => {
            var err;
            try {
              return client != null ? client.write($.TNET.stringify([msg, obj])) : void 0;
            } catch (error) {
              err = error;
              return typeof cb === "function" ? cb(err, false) : void 0;
            }
          };
          if (msg.f) {
            Output.setOutput(msg.f, (err, acted) => {
              return acted && send(msg.f);
            });
          }
          if (msg.d) {
            Output.setOutput(null, (err, acted) => {
              return acted && send(msg.d);
            });
          }
          if (msg.p) {
            outputFile = Output.getOutputFile();
            if (exists(outputFile)) {
              ChildProcess.spawn(`echo Log purged at \`date\`> ${expandPath(outputFile)}`, {
                shell: true
              }).on('exit', () => {
                return send(msg.p);
              });
            }
          }
          if (client != null) {
            if (msg.l) {
              send(Output.getOutputFile());
            }
            if (msg.t) {
              send(null); // signal the other side to init the stream
              handler = (data) => {
                var err, ref;
                try {
                  return client.write($.TNET.stringify([
                    {
                      t: true
                    },
                    String(data)
                  ]));
                } catch (error) {
                  err = error;
                  echo("tail socket error:", (ref = err.stack) != null ? ref : err);
                  return detach();
                }
              };
              detach = () => {
                return Output.stream.removeListener('tail', handler);
              };
              client.on('close', detach);
              client.on('error', detach);
              Output.stream.on('tail', handler);
            }
          }
          ret = msg.f || msg.d;
          if (ret) {
            if (typeof saveConfig === "function") {
              saveConfig();
            }
          }
          if (typeof cb === "function") {
            cb(null, ret);
          }
          return ret;
        },
        onResponse: function(item, socket) {
          var file, j, len, msg, resp;
          [msg, resp] = item;
          if (msg.f) {
            echo("Log file set:", msg.f);
          }
          if (msg.d) {
            echo("Log file disabled.");
          }
          if (msg.l) {
            echo('Output files:');
            for (j = 0, len = resp.length; j < len; j++) {
              file = resp[j];
              echo(file);
            }
          }
          if (msg.t) {
            if (resp) {
              process.stdout.write(resp);
            } else {
              echo("Connecting to log tail...");
            }
          } else {
            socket.end();
          }
          return false;
        }
      },
      // Add/remove a health check.
      health: {
        options: [["--group <group>", "Check all processes in this group."], ["--path <path>", "Will request http://localhost:port/<path> and check the response."], ["--status <code>", "Check the status code of the response."], ["--contains <text>", "Check that the response contains some bit of text."], ["--interval <secs>", "How often to run a check."], ["--timeout <ms>", "Fail if response is slower than this."], ["--delete", "Remove a health check."], ["--pause", "Temporarily pause a health check."], ["--resume", "Resume a health check after pausing."], ["--list", "List all current health checks."]],
        toMessage: function(cmd) {
          return {
            c: 'health',
            g: cmd.group,
            p: cmd.path,
            s: int(cmd.status),
            v: 1000 * int(cmd.interval),
            o: int(cmd.timeout),
            t: cmd.contains,
            d: trueFalse(cmd.delete),
            z: trueFalse(cmd.pause),
            r: trueFalse(cmd.resume),
            l: trueFalse(cmd.list)
          };
        },
        onMessage: function(msg, client, cb) {
          var reply;
          reply = function(x, ret) {
            if (client != null) {
              client.write($.TNET.stringify(x));
            }
            if (typeof cb === "function") {
              cb(ret);
            }
            return ret;
          };
          if (msg.l) { // --list
            return reply(Health.toConfig(), false);
          }
          if (!('g' in msg)) {
            return reply("Group is required.", false);
          }
          if (!Groups.has(msg.g)) {
            return reply(`No such group: ${msg.g}`, false);
          }
          if (msg.d) { // --delete
            Health.unmonitor(Groups.get(msg.g));
            return reply(`Will stop monitoring ${msg.g}.`, true);
          }
          if (msg.z) { // --pause
            if (Health.pause(msg.g)) {
              return reply(`Pausing monitor of ${msg.g}.`, true);
            } else {
              return reply("Group is not currently monitored.", false);
            }
          }
          if (msg.r) { // --resume
            if (Health.resume(msg.g)) {
              return reply(`Resuming monitor of ${msg.g}.`, true);
            } else {
              return reply("Group does not have a resumable monitor.", false);
            }
          }
          if (!('p' in msg)) {
            return reply("--path is required when adding a monitor.", false);
          }
          if (Health.monitor(Groups.get(msg.g), msg.p, msg.v, msg.s, msg.t, msg.o)) {
            return reply(`Adding monitor for ${msg.g}.`, true);
          } else {
            return reply("Did not add monitor.", false);
          }
        },
        onResponse: echoResponse
      },
      // Configure nginx integration.
      nginx: {
        options: [["--file <file>", "Auto-generate an nginx file with an upstream definition for each group."], ["--reload <cmd>", "What command to run in order to cause nginx to reload."], ["--disable", "Don't generate files or reload nginx."], ["--enable", "Generate a config file and reload nginx upon state changes."], ["--group <name>", "Optional. If given, the following options apply:"], ["--port <n>", "Default 80 (or 443). 'server { listen <n>; }'"], ["--name <hostname>", "Default is the group name. 'server { server_name <name>; }'"], ["--ssl_cert <path>", "Optional. If given, port defaults to 443."], ["--ssl_key <path>", "Optional. If given, port defaults to 443."], ["--list", "Show the current nginx configuration."]],
        toMessage: function(cmd) {
          return {
            c: 'nginx',
            f: cmd.file,
            r: cmd.reload,
            k: cmd.keepalive,
            d: trueFalse(cmd.disable),
            e: trueFalse(cmd.enable),
            l: cmd.list,
            g: cmd.group,
            p: cmd.port,
            n: cmd.name,
            sc: cmd.ssl_cert,
            sk: cmd.ssl_key
          };
        },
        onMessage: function(msg, client, cb) {
          var bytes, group, ref, ref1, reply;
          reply = function(m, ret) {
            if (ret) {
              if (client != null) {
                client.write($.TNET.stringify(m));
              }
              if (typeof saveConfig === "function") {
                saveConfig();
              }
            }
            if (typeof cb === "function") {
              cb(!ret && m || null, ret);
            }
            return ret;
          };
          if (msg.g != null) {
            if (!Groups.has(msg.g)) {
              return reply("No such group.", false);
            }
            group = Groups.get(msg.g);
            if (msg.p != null) {
              group.public_port = parseInt(msg.p, 10);
            }
            if (msg.n != null) {
              group.public_name = msg.n;
            }
            if (msg.sc != null) {
              if (!exists(msg.sc)) {
                return reply(`ssl_cert file '${msg.sc}' does not exist.`, false);
              }
              group.ssl_cert = msg.sc;
            }
            if (msg.sk != null) {
              if (!exists(msg.sk)) {
                return reply(`ssl_key file '${msg.sk}' does not exist.`, false);
              }
              group.ssl_key = msg.sc;
            }
            return reply("Group updated.", true);
          }
          if (msg.l != null) {
            return reply(Nginx.toConfig(), false);
          }
          if ((ref = msg.f) != null ? ref.length : void 0) {
            Nginx.setFile(msg.f);
          }
          if ((ref1 = msg.r) != null ? ref1.length : void 0) {
            Nginx.setReload(msg.r);
          }
          if (msg.e) {
            Nginx.setDisabled(false);
            if (!exists(nginxTemplate)) {
              bytes = Nginx.defaults.templateText;
              Fs.writeFile(expandPath(nginxTemplate), bytes, (err, written) => {
                if (err) {
                  return warn(err);
                } else if (written !== bytes.length) {
                  return warn(`Only wrote ${written} out of ${bytes.length} to ${nginxTemplate}`);
                } else {
                  return echo(`Wrote default content to ${nginxTemplate}`);
                }
              });
            }
          }
          if (msg.d) {
            Nginx.setDisabled(true);
          }
          if (typeof saveConfig === "function") {
            saveConfig();
          }
          return reply("nginx configuration updated.", true);
        },
        onResponse: echoResponse
      },
      config: {
        options: [["--purge", "Remove all configuration."], ["--list", "Show the current configuration."]],
        toMessage: function(cmd) {
          return {
            c: 'config',
            p: trueFalse(cmd.purge),
            l: trueFalse(cmd.list)
          };
        },
        onMessage: function(msg, client, cb) {
          if (msg.p) {
            Fs.writeFile(expandPath(configFile), "", cb);
            if (client != null) {
              client.write($.TNET.stringify("Cleared log file."));
            }
          } else {
            Fs.readFile(expandPath(configFile), function(err, data) {
              if (err) {
                return;
              }
              return client != null ? client.write($.TNET.stringify(String(data))) : void 0;
            });
          }
          return false;
        },
        onResponse: echoResponse
      }
    }
  });

  ({saveConfig} = require('./util/config'));

}).call(this);
