// Generated by CoffeeScript 2.2.3
(function() {
  //!/usr/bin/env coffee
  var $, Daemon, Fs, basePath, cmd, configFile, createBasePath, doInit, echo, exists, exit_soon, expandPath, sendServerCmd, socketFile, verbose, warn;

  ({$, cmd, echo, warn, verbose, exit_soon} = require('../common'));

  Fs = require('fs');

  ({exists, socketFile, configFile, basePath, createBasePath, expandPath} = require('../files'));

  Daemon = require('../daemon');

  if (cmd.help || cmd.h || cmd._[0] === 'help') {
    echo("shepherd <start|stop|restart|status|add|remove|enable|disable>");
    if (cmd.verbose) {
      echo("\n- start [id] : start an instance, group, or all (if no id given)\n- stop [id] : stop an instance, group, or all (if no id given)\n- restart [id] : restart an instance, group, or all (if no id given)\n- status : report status of everything\n- add [name] [--cd 'path'] <--exec 'command'> [--count n] [--port p] : adds a new process group\n  --cd 'path' - Optional, working directory of processes in the group.\n  --exec 'command' - Required. The shell command to execute, will be parsed by /bin/sh.\n  --count N - Optional. Number of processes to spawn. Default 1.\n  --port P - Optional. Assign the PORT environment variable for each process, incrementing from P.\n    A process given --port is \"started\" only when it starts listening on the proper PORT.\n- remove [name] : remove a process group\n- enable [id] : enable a group or single process\n- disable [id] : disable a group or single process\n\ne.g. 'shep add mygroup --cd test --exec \"node app.js\" --count 3 --port 1080'\ne.g. 'shep disable mygroup-2' # just disable the 3rd member of the group\n");
    }
    process.exit(0);
  }

  doInit = function(cb) {
    var defaultsFile;
    defaultsFile = process.cwd() + "/.shep/defaults";
    if (exists(configFile) && !(cmd.f || cmd.force)) {
      echo(`Configuration already exists (${configFile})`);
      if (typeof cb === "function") {
        cb(null, false);
      }
    } else {
      verbose("Checking for defaults file:", defaultsFile);
      if (exists(defaultsFile)) {
        echo("Applying default config...");
        Fs.copyFile(expandPath(defaultsFile), expandPath(configFile), (err) => {
          return typeof cb === "function" ? cb(null, true) : void 0;
        });
      } else {
        createBasePath(".", cb);
      }
    }
    return null;
  };

  sendServerCmd = (_cmd, cb) => {
    var Actions, Net, Tnet, action, on_error, retryConnect;
    if (!exists(basePath)) {
      return echo("No .shep directory found.");
    }
    if (!exists(socketFile)) {
      return echo("Status: offline.");
    }
    ({Actions} = require('../actions'));
    if (!(action = Actions[_cmd])) {
      return warn("No such action:", _cmd);
    }
    Net = require('net');
    Tnet = require('../util/tnet');
    on_error = (err) => {
      return warn("socket error", $.debugStack(err));
    };
    (retryConnect = () => {
      var connectStart, socket;
      connectStart = Date.now();
      socket = Net.connect({
        path: expandPath(socketFile)
      });
      socket.on('close', () => {
        return typeof cb === "function" ? cb(null, true) : void 0;
      });
      socket.on('error', (err) => { // probably daemon is not running, should start it
        if (err.code === 'ENOENT') {
          if (_cmd === 'start') {
            Daemon.doStart(false);
            return setTimeout(retryConnect, 3000);
          } else {
            echo("Status: offline.");
            return exit_soon(1);
          }
        } else if (err.code === 'EADDRNOTAVAIL') {
          echo("Status: offline.");
          return exit_soon(1);
        } else {
          return on_error(err);
        }
      });
      socket.on('connect', () => {
        var bytes, err, msg;
        socket._connectLatency = Date.now() - connectStart;
        try {
          msg = action.toMessage(cmd);
          if (cmd._.length > 1 && !(('group' in cmd) || ('instance' in cmd))) {
            if (/\w/.test(cmd._[1])) {
              msg.auto = cmd._[1];
            }
          }
          bytes = $.TNET.stringify(msg);
        } catch (error) {
          err = error;
          return on_error(err);
        }
        socket.write(bytes, () => {
          var timeout;
          if (typeof action.onConnect === "function") {
            action.onConnect(socket);
          }
          if ('onResponse' in action) {
            timeout = $.delay(1000, () => {
              warn("Timed-out waiting for a response from the daemon.");
              return socket.end();
            });
            Tnet.read_stream(socket, (item) => {
              timeout.cancel();
              return action.onResponse(item, socket);
            });
          } else {
            socket.end();
          }
          return null;
        });
        return null;
      });
      return null;
    })();
    return null;
  };

  switch (cmd._[0]) {
    case 'init':
      return doInit(exit_soon);
    case 'up':
      Daemon.doStart(false);
      $.delay(1000, () => {
        return sendServerCmd('status');
      });
      break;
    case 'down':
      return Daemon.doStop(true);
    default:
      sendServerCmd(cmd._[0], () => {
        var ref;
        if ((ref = cmd._[0]) === 'start' || ref === 'stop' || ref === 'enable' || ref === 'disable' || ref === 'add' || ref === 'remove' || ref === 'scale' || ref === 'replace') {
          return setTimeout((() => {
            return sendServerCmd('status', () => {
              return exit_soon(0);
            });
          }), 300);
        } else {
          return exit_soon(0);
        }
      });
  }

}).call(this);
