// Generated by CoffeeScript 2.2.3
(function() {
  var $, ChildProcess, Group, Groups, Nginx, Path, Proc, Shell, SlimProcess, actOnAll, actOnGroup, actOnInstance, addGroup, afterAction, echo, exists, quoted, removeGroup, saveConfig, simpleAction, verbose, warn;

  ({$, echo, warn, verbose} = require('../common'));

  Path = require('path');

  Nginx = null; // placeholder for out of order import

  saveConfig = null;

  Shell = require('shelljs');

  ChildProcess = require('child_process');

  SlimProcess = require('../util/process-slim');

  ({exists} = require('../files'));

  // the global herd of processes
  Groups = new Map();

  // m.clear m.delete m.entries m.forEach m.get m.has m.keys m.set m.size m.values
  const DEFAULT_GRACE = 9000;;

  quoted = function(s) {
    return '"' + s.replace(/"/g, '\\"') + '"';
  };

  Group = (function() {
    var createProcess;

    class Group extends Array {
      constructor(name1, cd1, exec1, n1, port1, grace1 = DEFAULT_GRACE) {
        var i, j, ref;
        super();
        this.name = name1;
        this.cd = cd1;
        this.exec = exec1;
        this.n = n1;
        this.port = port1;
        this.grace = grace1;
        for (i = j = 0, ref = this.n; 1 !== 0 && (1 > 0 ? j < ref : j > ref); i = j += 1) {
          this.push(createProcess(this, i));
        }
        this.monitors = Object.create(null); // used later by health checks
        this.enabled = true;
      }

      enable(cb) {
        var acted;
        acted = !this.enabled;
        this.enabled = true;
        if (acted) {
          return this.actOnAll('enable', cb);
        } else {
          return typeof cb === "function" ? cb() : void 0;
        }
      }

      disable(cb) {
        var acted;
        acted = this.enabled;
        this.enabled = false;
        if (acted) {
          return this.actOnAll('disable', cb);
        } else {
          return typeof cb === "function" ? cb() : void 0;
        }
      }

      scale(n, cb) {
        var d, dn, j, p, progress, ref;
        dn = n - this.n;
        progress = $.Progress(dn);
        progress.then(() => {
          return saveConfig(cb);
        });
        if (dn > 0) {
          echo(`[scale(${n})] Adding ${dn} instances...`);
          for (d = j = 0, ref = dn; 1 !== 0 && (1 > 0 ? j < ref : j > ref); d = j += 1) {
            p = createProcess(this, this.n++);
            if (this.enabled) {
              p.start(() => {
                return progress.finish(1);
              });
            } else {
              p.disable(() => {
                return progress.finish(1);
              });
            }
            this.push(p);
          }
        } else if (dn < 0) {
          echo(`[scale(${n})] Trimming ${dn} instances...`);
          while (this.n > n) {
            this.pop().stop(() => {
              return progress.finish(1);
            });
            this.n -= 1;
          }
        } else {
          return false;
        }
        return true;
      }

      restart(cb) { // Slowly kill and restart one at a time
        var oneStep;
        (oneStep = (i) => {
          if (!(i < this.length)) {
            return typeof cb === "function" ? cb() : void 0;
          }
          return this[i].restart(() => {
            return oneStep(i + 1);
          });
        })(0);
        return true;
      }

      start(cb) {
        var oneStep;
        (oneStep = (i) => {
          if (!(i < this.length)) {
            return typeof cb === "function" ? cb(null, true) : void 0;
          }
          return this[i].start(() => {
            return oneStep(i + 1);
          });
        })(0);
        return true;
      }

      stop(cb) {
        var j, k, len, len1, proc, ref, ref1;
        ref = this;
        for (j = 0, len = ref.length; j < len; j++) {
          proc = ref[j];
          proc.expected = false;
        }
        ref1 = this;
        for (k = 0, len1 = ref1.length; k < len1; k++) {
          proc = ref1[k];
          proc.stop();
        }
        return typeof cb === "function" ? cb(null, true) : void 0;
      }

      markAsInvalid(reason) {
        var j, len, proc, ref;
        ref = this;
        for (j = 0, len = ref.length; j < len; j++) {
          proc = ref[j];
          proc.markAsInvalid(reason);
        }
        return this;
      }

      actOnAll(method, cb) {
        var j, len, progress, ref, x;
        progress = $.Progress(this.length);
        progress.wait((err) => {
          return cb(err, true);
        });
        ref = this;
        for (j = 0, len = ref.length; j < len; j++) {
          x = ref[j];
          x[method](() => {
            return progress.finish(1);
          });
        }
        return progress;
      }

      toString() {
        var i, proc;
        return `[group ${this.name}] ` + ((function() {
          var j, len, ref, results;
          ref = this;
          results = [];
          for (i = j = 0, len = ref.length; j < len; i = ++j) {
            proc = ref[i];
            results.push(`${(proc.port ? `(${proc.port}) ` : "")}${proc.statusString}`);
          }
          return results;
        }).call(this)).join(", ");
      }

      toConfig() {
        return `add --group ${this.name}` + (this.cd !== "." ? ` --cd ${quoted(this.cd)}` : "") + ` --exec ${quoted(this.exec)} --count ${this.n}` + (this.grace !== DEFAULT_GRACE ? ` --grace ${this.grace}` : "") + (this.port ? ` --port ${this.port}` : "");
      }

    };

    createProcess = function(g, i) {
      var port;
      port = void 0;
      if (g.port) {
        port = g.port + i;
      }
      return new Proc(`${g.name}-${i}`, g.cd, g.exec, port, g);
    };

    return Group;

  }).call(this);

  Proc = (function() {
    class Proc {
      constructor(id, cd1, exec1, port1, group1) {
        var statusString;
        this.id = id;
        this.cd = cd1;
        this.exec = exec1;
        this.port = port1;
        this.group = group1;
        // the time of the most recent start
        this.started = false;
        this.enabled = true;
        this.healthy = void 0; // used later by health checks
        this.cooldown = Proc.cooldown; // this increases after each failed restart
        this.expected = false; // should we restart this proc if it exits
        // expose uptime
        $.defineProperty(this, 'uptime', {
          get: () => {
            if (this.started) {
              return $.now - this.started;
            } else {
              return 0;
            }
          }
        });
        statusString = "unstarted";
        $.defineProperty(this, 'statusString', {
          get: () => {
            return statusString;
          },
          set: (v) => {
            var oldValue;
            oldValue = statusString;
            statusString = v;
            // @log oldValue + " -> " + v
            return $.log(this.group.toString());
          }
        });
      }

      log(...args) {
        return $.log(`[${this.id}]`, ...args);
      }

      // Start this process if it isn't already.
      start(cb) { // cb called with a 'started' flag that indicates if any work was done
        var clearPort, doStart, done, env, retryStart;
        done = (err, ret) => {
          if (typeof cb === "function") {
            cb(err, ret);
          }
          return ret;
        };
        if (this.started || !this.enabled) {
          verbose("Not starting because", {started: this.started, enabled: this.enabled});
          return done(null, false);
        }
        this.expected = true;
        env = Object.assign({}, process.env, {
          PORT: this.port
        });
        clearPort = (cb) => {
          if (!(this.expected && this.enabled)) {
            verbose("Giving up on clearPort because", this.expected, this.enabled);
            return done(null, false); // stopped while waiting
          }
          if (this.port) {
            return SlimProcess.getPortOwner(this.port, (err, owner) => {
              var invalidPort;
              if (!owner) {
                return cb();
              }
              invalidPort = () => {
                return done(this.markAsInvalid("invalid port"));
              };
              if (owner.uid !== process.getuid()) {
                return invalidPort();
              }
              return SlimProcess.getProcessTable((err, procs) => {
                SlimProcess.visitProcessTree(process.pid, (proc) => {
                  if (proc.pid === owner.pid) {
                    invalidPort();
                  }
                  return null;
                });
                if (this.statusString !== "invalid port") {
                  this.statusString = `killing ${owner.pid}`;
                  try {
                    process.kill(owner.pid, 'SIGTERM');
                  } catch (error) {}
                  return setTimeout((() => {
                    return clearPort(cb);
                  }), 1000);
                }
              });
            });
          } else {
            return typeof cb === "function" ? cb() : void 0;
          }
        };
        retryStart = () => {
          if (this.started || !this.enabled) {
            return done(null, false);
          }
          this.cooldown = Math.min(30000, this.cooldown * 2);
          this.statusString = `waiting ${this.cooldown}`;
          setTimeout(doStart, this.cooldown);
          return true;
        };
        return (doStart = () => {
          if (this.started || !this.expected) {
            verbose("Giving up on doStart because", this.expected, this.enabled);
            return done(null, false);
          }
          return clearPort((err) => {
            var _s, checkStarted, finishStarting, opts;
            if (err || !this.expected) {
              verbose("Giving up after clearPort because", err, this.expected);
              return done(null, false);
            }
            checkStarted = null;
            this.statusString = "starting";
            try {
              this.cd = Path.resolve(process.cwd(), this.cd);
              verbose("cd:", this.cd);
            } catch (error) {
              err = error; // it's actually possible for node's internals to throw an exception here if cwd() is weird
              this.markAsInvalid(err.message);
              return done(err, false);
            }
            verbose("exec:", this.exec, "as", this.id);
            opts = {
              shell: true,
              cwd: this.cd,
              env: env
            };
            if (!exists(this.cd)) {
              return this.stop(() => {
                return this.group.markAsInvalid("invalid dir");
              });
            }
            this.proc = ChildProcess.spawn(this.exec, opts);
            this.expected = true; // tell the 'exit' handler to bring us back up if we die
            finishStarting = () => {
              this.started = $.now;
              this.cooldown = Proc.cooldown;
              this.statusString = "started";
              if (this.port) {
                return Nginx.sync(() => {
                  return done(true);
                });
              } else {
                return done(true);
              }
            };
            if (this.port) {
              _s = Date.now();
              checkStarted = setTimeout((() => {
                var ref;
                if (!((this.proc != null) && this.expected)) {
                  return done(null, false);
                }
                if (((ref = this.proc) != null ? ref.pid : void 0) == null) {
                  this.statusString = "exec failed";
                  this.started = this.expected = this.enabled = this.healthy = false;
                  return done(null, false);
                }
                echo(`Waiting for port ${this.port} to be owned by ${this.proc.pid} (will wait ${this.group.grace} ms)`);
                return SlimProcess.waitForPortOwner(this.proc, this.port, this.group.grace, (err, owner) => {
                  if (!(this.expected && this.enabled)) { // stopped while waiting?
                    echo("Giving up after waiting for port because", this.expected, this.enabled);
                    return this.stop(() => {
                      return done(null, false);
                    });
                  }
                  switch (err) {
                    case 'exit':
                      warn("Process exited immediately, will not retry.");
                      this.proc = null;
                      return this.stop(() => {
                        return this.disable(() => {
                          this.statusString = "failed";
                          return done(null, false);
                        });
                      });
                    case 'timeout':
                      echo(`Port ${this.port} was not owned within the timeout: ${this.group.grace}ms`);
                      return this.stop(() => {
                        return done(null, false);
                      });
                    default:
                      if (err != null) {
                        echo("Failed to find port owner,", err, "after", (Date.now() - _s) + "ms");
                        return this.stop(retryStart);
                      }
                  }
                  return finishStarting();
                });
              }), 50); // if there is no port to wait for then staying up for a few seconds counts as started
            } else {
              checkStarted = setTimeout(finishStarting, this.grace);
            }
            
            // Connect the process output to our log writer.
            this.proc.stdout.on('data', (data) => {
              return this.log(data.toString("utf8"));
            });
            this.proc.stderr.on('data', (data) => {
              return this.log("(stderr)", data.toString("utf8"));
            });
            this.proc.on('exit', (code, signal) => {
              var ref, ref1;
              clearTimeout(checkStarted);
              this.statusString = `exit(${code})` + (this.expected ? " expected" : "");
              this.started = false;
              if (this.expected) {
                echo("Exit was not expected, restarting...");
                return retryStart();
              }
              if ((ref = this.proc) != null ? ref.pid : void 0) {
                try {
                  if ((ref1 = this.proc) != null) {
                    if (typeof ref1.unref === "function") {
                      ref1.unref();
                    }
                  }
                } catch (error) {}
              }
              return this.proc = void 0;
            });
            return true;
          });
        })();
      }

      stop(cb) {
        var ref;
        this.statusString = "stopping";
        this.expected = false;
        if (((ref = this.proc) != null ? ref.pid : void 0) > 1) {
          this.proc.on('exit', () => {
            this.started = false;
            this.statusString = this.enabled ? "stopped" : "disabled";
            if (this.port) {
              return Nginx.sync(() => {
                return typeof cb === "function" ? cb(null, true) : void 0;
              });
            } else {
              return typeof cb === "function" ? cb(null, true) : void 0;
            }
          });
          try {
            process.kill(this.proc.pid);
          } catch (error) {}
          return true;
        }
        this.started = false;
        this.proc = null;
        this.statusString = this.enabled ? "stopped" : "disabled";
        if (typeof cb === "function") {
          cb(null, false);
        }
        return false;
      }

      restart(cb) {
        this.statusString = "restarting";
        return this.stop(() => {
          return this.start(cb);
        });
      }

      enable(cb) {
        var acted;
        acted = !this.enabled;
        this.enabled = true;
        if (acted) {
          this.start(cb);
        } else {
          if (typeof cb === "function") {
            cb();
          }
        }
        return acted;
      }

      markAsInvalid(reason) {
        this.enabled = this.expected = this.started = this.healthy = false;
        this.statusString = reason;
        return false;
      }

      disable(cb) {
        var _end, acted;
        acted = this.started || this.enabled;
        this.enabled = false;
        _end = () => {
          this.statusString = "disabled";
          return typeof cb === "function" ? cb() : void 0;
        };
        if (acted) {
          this.stop(_end);
        } else {
          _end();
        }
        return acted;
      }

    };

    Proc.cooldown = 200;

    return Proc;

  }).call(this);

  actOnInstance = function(method, instanceId, cb) {
    var acted, chunks, groupId, index, proc, ref;
    if (!(instanceId != null ? instanceId.length : void 0)) {
      return false;
    }
    acted = false;
    chunks = instanceId.split('-');
    index = chunks[chunks.length - 1];
    groupId = chunks.slice(0, chunks.length - 1).join('-');
    index = parseInt(index, 10);
    proc = (ref = Groups.get(groupId)) != null ? ref[index] : void 0;
    if ((!proc) || !(method in proc)) {
      return typeof cb === "function" ? cb('invalid method') : void 0;
    }
    proc[method]((ret) => {
      return afterAction(method, ret, cb);
    });
    return false;
  };

  actOnGroup = function(method, groupId, cb) {
    var group;
    group = Groups.get(groupId);
    if (group == null) {
      return false;
    }
    if (!group) {
      return typeof cb === "function" ? cb('invalid group') : void 0;
    }
    if (method in group) {
      group[method]((ret) => {
        return afterAction(method, ret, cb);
      });
    } else {
      group.actOnAll(method, (ret) => {
        return afterAction(method, ret, cb);
      });
    }
    return false;
  };

  actOnAll = function(method, cb) {
    var acted, progress;
    acted = false;
    progress = $.Progress(Groups.size + 1).then(() => {
      return afterAction(method, acted, cb);
    });
    Groups.forEach(function(group) {
      var j, len, proc, results;
      if ('function' === typeof group[method]) {
        return group[method]((ret) => {
          acted = ret || acted;
          return progress.finish(1);
        });
      } else {
        results = [];
        for (j = 0, len = group.length; j < len; j++) {
          proc = group[j];
          if (method in proc) {
            results.push(proc[method]((ret) => {
              acted = ret || acted;
              return progress.finish(1);
            }));
          }
        }
        return results;
      }
    });
    return progress.finish(1);
  };

  addGroup = function(name, cd = ".", exec, count = 1, port, grace = DEFAULT_GRACE, cb) {
    if (Groups.has(name)) {
      echo("Group already exists. Did you mean 'replace'?");
      return false;
    }
    echo("Adding group:", name);
    verbose({cd, exec, count, port, grace});
    Groups.set(name, new Group(name, cd, exec, count, port, grace));
    return afterAction('add', true, cb);
  };

  removeGroup = function(name, cb) {
    var done, g, j, len, proc;
    if (!Groups.has(name)) {
      echo("No such group:", name);
      return false;
    }
    echo("Removing group:", name);
    g = Groups.get(name);
    done = $.Progress(g.length).then(() => {
      Groups.delete(name);
      return afterAction('remove', true, cb);
    });
    for (j = 0, len = g.length; j < len; j++) {
      proc = g[j];
      proc.stop(() => {
        return done.finish(1);
      });
    }
    return true;
  };

  afterAction = function(method, ret, cb) {
    if (method === 'enable' || method === 'disable' || method === 'add' || method === 'remove' || method === 'replace') {
      return saveConfig((err, acted) => {
        return typeof cb === "function" ? cb(err, ret) : void 0;
      });
    } else {
      return typeof cb === "function" ? cb(null, ret) : void 0;
    }
  };

  simpleAction = function(method) {
    return function(msg, client, cb) {
      var _line, targetText;
      _line = '';
      targetText = msg.g ? `group ${msg.g}` : msg.i ? `instance ${msg.i}` : "everything";
      switch (method) {
        case 'start':
          _line += `Starting ${targetText}...`;
          break;
        case 'stop':
          _line += `Stopping ${targetText}...`;
          break;
        case 'restart':
          _line += `Restarting ${targetText}...`;
          break;
        case 'enable':
          _line += `Enabling ${targetText}...`;
          break;
        case 'disable':
          _line += `Disabling ${targetText}...`;
      }
      if (_line.length > 0) {
        echo(_line);
        if (client != null) {
          client.write($.TNET.stringify(_line));
        }
      } else {
        echo("simpleAction", method, JSON.stringify(msg));
      }
      switch (false) {
        case !msg.g:
          return actOnGroup(method, msg.g, cb);
        case !msg.i:
          return actOnInstance(method, msg.i, cb);
        default:
          return actOnAll(method, cb);
      }
    };
  };

  Object.assign(module.exports, {Groups, actOnAll, actOnInstance, addGroup, removeGroup, simpleAction});

  // fulfill some out-of-order obligations
  Nginx = require('./nginx');

  ({saveConfig} = require('../util/config'));

}).call(this);
