// Generated by CoffeeScript 2.2.3
(function() {
  //!/usr/bin/env coffee
  // this is the master process that maintains the herd of processes
  var $, Actions, Chalk, ChildProcess, Fs, Groups, Net, Output, Shell, _c, basePath, cmd, configFile, die, doStart, doStatus, doStop, echo, exists, exit_soon, expandPath, handleMessage, outputFile, pidFile, readConfig, readPid, runDaemon, socketFile, started, verbose, warn;

  ({$, cmd, echo, warn, verbose} = require('../common'));

  Fs = require('fs');

  Net = require('net');

  Shell = require('shelljs');

  Chalk = require('chalk');

  Output = require('./output');

  ({Groups} = require('./groups'));

  ({Actions} = require('../actions'));

  ({pidFile, socketFile, outputFile, configFile, basePath, expandPath} = require('../files'));

  ({readConfig} = require('../util/config'));

  ChildProcess = require('child_process');

  exit_soon = (n, ms = 200) => {
    return setTimeout((() => {
      return process.exit(n);
    }), ms);
  };

  die = function(...msg) {
    console.error(...msg);
    return exit_soon(1);
  };

  if (!('HOME' in process.env)) {
    echo("No $HOME in environment, can't place .shep directory.");
    return exit_soon(1);
  }

  readPid = function() {
    try {
      return parseInt(Fs.readFileSync(expandPath(pidFile)).toString(), 10);
    } catch (error) {
      return void 0;
    }
  };

  handleMessage = function(msg, client, cb) {
    var g, i, ref;
    if ('auto' in msg) {
      if (Groups.has(msg.auto)) {
        msg.g = msg.auto;
      } else {
        [g, i] = msg.auto.split('-');
        if (Groups.has(g)) {
          msg.i = g + "-" + parseInt(i, 10);
        } else {
          msg.g = msg.auto;
        }
      }
    }
    return (ref = Actions[msg.c]) != null ? typeof ref.onMessage === "function" ? ref.onMessage(msg, client, cb) : void 0 : void 0;
  };

  exists = function(path) {
    var stat;
    try {
      return (stat = Fs.statSync(expandPath(path))).isFile() || stat.isSocket();
    } catch (error) {
      return false;
    }
  };

  doStop = function(exit) {
    var pid, result;
    if (pid = readPid()) {
      Actions.stop.onMessage({}); // send a stop command to all running instances
      // give them a little time to exit gracefully
      // then kill the pid from the pid file (our own?)
      result = Shell.exec(`kill ${pid}`, {
        silent: true,
        async: false // use Shell.exec for easier stderr peek after
      });
      if (result.stderr.indexOf("No such process") > -1) {
        try {
          Fs.unlinkSync(expandPath(pidFile));
        } catch (error) {}
        try {
          Fs.unlinkSync(expandPath(socketFile));
        } catch (error) {}
      }
    }
    echo("Status: offline.");
    if (exit) {
      return exit_soon(0);
    }
  };

  doStatus = function() {
    $.log("Socket:", socketFile, exists(socketFile) ? Chalk.green("(exists)") : Chalk.yellow("(does not exist)"));
    return $.log("PID File:", pidFile, exists(pidFile) ? Chalk.green("(exists)") : Chalk.yellow("(does not exist)"));
  };

  started = false;

  runDaemon = () => { // in the foreground
    var _pidFile, _socketFile;
    if (!(pidFile && socketFile)) {
      return die("Daemon does not have a base path.");
    }
    if (exists(pidFile)) {
      return die("Already running as PID:" + readPid());
    }
    if (exists(socketFile)) {
      return die("Socket file still exists:" + socketFile);
    }
    _pidFile = expandPath(pidFile);
    _socketFile = expandPath(socketFile);
    return Fs.writeFile(_pidFile, process.pid, (err) => {
      if (err) {
        return die("Failed to write pid file:", err);
      }
      return Output.setOutput(outputFile, (err) => {
        var j, len, ref, shutdown, sig, socket;
        if (err) {
          return die("Failed to set output file:", err);
        }
        echo("Opening master socket...", socketFile);
        socket = Net.Server().listen({
          path: _socketFile
        });
        socket.on('error', function(err) {
          echo("Failed to open local socket:", $.debugStack(err));
          return exit_soon(1);
        });
        socket.on('connection', function(client) {
          client.on('error', function(err) {
            return warn("client error:", err);
          });
          return client.on('data', function(msg) {
            var start;
            start = Date.now();
            msg = $.TNET.parse(msg.toString());
            return handleMessage(msg, client, (err, acted) => {
              var _msg, k, v;
              if (err) {
                try {
                  warn(msg, "caused", $.debugStack(err));
                  return client != null ? client.write($.TNET.stringify($.debugStack(err))) : void 0;
                } catch (error) {}
              }
              _msg = Object.create(null);
              for (k in msg) {
                v = msg[k];
                if (v != null) {
                  _msg[k] = v;
                }
              }
              return echo("Command handled:", _msg, "in", Date.now() - start, "ms");
            });
          });
        });
        shutdown = function(signal) {
          return function() {
            try {
              echo("Shutting down...", signal);
            } catch (error) {}
            try {
              socket.close();
            } catch (error) {}
            Groups.forEach(function(group) { // TODO: Groups.shutdown()
              var j, len, proc;
              for (j = 0, len = group.length; j < len; j++) {
                proc = group[j];
                proc.expected = false;
              }
              return null;
            });
            Groups.forEach(function(group) {
              return group.stop();
            });
            try {
              Fs.unlinkSync(_pidFile);
            } catch (error) {}
            if (signal !== 'exit') {
              return exit_soon(0);
            }
            return null;
          };
        };
        ref = ['SIGINT', 'SIGTERM', 'exit'];
        for (j = 0, len = ref.length; j < len; j++) {
          sig = ref[j];
          process.on(sig, shutdown(sig));
        }
        readConfig();
        return started = true;
      });
    });
  };

  doStart = (exit) => { // launch the daemon in the background and exit
    var _cmd, child, devNull, stdio;
    echo("Starting daemon...");
    _cmd = process.argv.slice(0, 2).join(' ').replace("client/index", "daemon/index") + " daemon" + ` --base "${basePath.replace(/\/.shep$/, '')}"` + (cmd.verbose ? " --verbose" : "") + (cmd.quiet ? " --quiet" : "");
    devNull = Fs.openSync("/dev/null", 'a+');
    stdio = [
      devNull,
      devNull,
      process.stderr // only let stderr pass through
    ];
    child = ChildProcess.spawn(_cmd, {
      detached: true,
      shell: true,
      stdio: stdio
    });
    child.on('error', function(err) {
      return console.error("Child exec error:", err);
    });
    child.unref();
    if (exit) {
      exit_soon(0);
    }
    return true;
  };

  if (require.main === module) {
    switch (_c = cmd._[0]) {
      case "stop":
        doStop(true);
        console.log("Stopped.");
        break;
      case "start":
        doStart(true);
        break;
      case "daemon":
        runDaemon();
        break;
      case "restart":
        doStop(false);
        doStart(true);
        break;
      case "status":
        doStatus();
        break;
      default:
        console.log("Unknown usage:", cmd);
        console.log("Usage: shepd <command>");
        console.log("Commands: start stop restart status help");
        exit_soon(0);
    }
  }

  true;

  Object.assign(module.exports, {doStart, doStop, doStatus});

}).call(this);
