// Generated by CoffeeScript 2.2.3
(function() {
  var $, ChildProcess, Files, Fs, Groups, Handlebars, defaults, disabled, echo, expandPath, generate, reload, reloadNginx, server_name, server_port, setDisabled, setFile, setReload, setTemplate, sync, toConfig, verbose, warn, writeNginxFile;

  ({$, echo, warn, verbose} = require('../common'));

  Fs = require('fs');

  Files = require('../files');

  ({Groups} = require('./groups'));

  Handlebars = require('handlebars');

  ChildProcess = require('child_process');

  ({expandPath} = Files);

  defaults = {
    reload: reload = "service nginx reload",
    server_name: server_name = null,
    server_port: server_port = 80,
    disabled: disabled = true,
    template: Files.nginxTemplate = "%/nginx.template",
    templateText: "upstream {{ name }} {\n{{#each group}}\n{{#if this.port}}\n	server 127.0.0.1:{{ this.port }} {{#unless this.started}}down{{/unless}};\n{{/if}}\n{{/each}}\n}\nserver {\n	listen {{ public_port }};\n	server_name {{ public_name }};\n	location / {\n		proxy_pass http://{{ name }};\n	}\n}\n{{#if ssl_cert}}\nserver {\n	listen 443;\n	server_name {{ public_name }};\n	ssl on;\n	ssl_certificate {{ ssl_cert }};\n	ssl_certificate_key {{ ssl_key }};\n	location / {\n		proxy_pass http://{{ name }};\n	}\n}\n{{/if}}"
  };

  setReload = function(r) {
    return reload = r;
  };

  setTemplate = function(f) {
    return Files.nginxTemplate = f;
  };

  setFile = function(f) {
    return Files.nginxFile = f.replace(/^~/, process.env.HOME).replace(/^%/, Files.basePath);
  };

  setDisabled = function(d) {
    return disabled = d;
  };

  generate = (t) => {
    var buf;
    buf = "";
    if (t == null) {
      return "no template";
    }
    Groups.forEach((group) => {
      var ref, ref1;
      if (group.port != null) {
        return buf += t({
          group: group,
          public_name: (ref = group.public_name) != null ? ref : group.name,
          public_port: (ref1 = group.public_port) != null ? ref1 : 80,
          ssl_cert: group.ssl_cert,
          ssl_key: group.ssl_key,
          name: group.name
        });
      }
    });
    return buf;
  };

  writeNginxFile = (cb) => {
    var start;
    if (disabled) {
      return typeof cb === "function" ? cb(null, false) : void 0;
    }
    start = Date.now();
    verbose("Reading nginx template...", Files.nginxTemplate);
    Fs.readFile(expandPath(Files.nginxTemplate), function(err, data) {
      var t, text;
      if (err) {
        warn("Failed to read nginx template:", err);
        return typeof cb === "function" ? cb(null, false) : void 0;
      }
      t = Handlebars.compile(data.toString('utf8'));
      text = generate(t);
      verbose("Saving nginx file...", Files.nginxFile);
      Fs.writeFile(expandPath(Files.nginxFile), text, function(err) {
        verbose(`writeNginxFile took ${Date.now() - start} ms`);
        return typeof cb === "function" ? cb(err, err == null) : void 0;
      });
      return null;
    });
    return null;
  };

  toConfig = () => {
    var _disabled, _file, _reload, _template, buf;
    _reload = reload === defaults.reload ? "" : ` --reload '${reload}'`;
    _file = Files.nginxFile === Files.makePath("nginx") ? "" : ` --file '${Files.nginxFile}'`;
    _template = Files.nginxTemplate === defaults.template ? "" : ` --template '${Files.nginxTemplate}'`;
    _disabled = disabled ? " --disable" : " --enable";
    buf = `nginx${_disabled}${_reload}${_file}${_template}`;
    Groups.forEach((group) => {
      if ((group.public_name != null) || (group.public_port != null) || (group.ssl_cert != null) || (group.ssl_key != null)) {
        return buf += `\nnginx --group ${group.name}` + (group.public_name ? ` --name ${group.public_name}` : "") + (group.public_port ? ` --port ${group.public_port}` : "") + (group.ssl_cert ? ` --ssl_cert ${group.ssl_cert}` : "") + (group.ssl_key ? ` --ssl_key ${group.ssl_key}` : "");
      }
    });
    return buf;
  };

  reloadNginx = (cb) => {
    var p;
    if (disabled) {
      return typeof cb === "function" ? cb(null, false) : void 0;
    }
    echo("Reloading nginx...");
    p = ChildProcess.exec(reload, {
      shell: true
    });
    p.stdout.on('data', function(data) {
      return $.log(`${reload}`, data.toString("utf8"));
    });
    p.stderr.on('data', function(data) {
      return $.log(`${reload} (stderr)`, data.toString("utf8"));
    });
    return typeof cb === "function" ? cb(null, false) : void 0;
  };

  sync = (cb) => {
    return writeNginxFile((err, acted) => {
      if (err || !acted) {
        return typeof cb === "function" ? cb(err, acted) : void 0;
      }
      return reloadNginx(cb);
    });
  };

  Object.assign(module.exports, {writeNginxFile, reloadNginx, setReload, setTemplate, setFile, setDisabled, toConfig, sync, defaults});

}).call(this);
