// Generated by CoffeeScript 2.2.3
(function() {
  var $, ChildProcess, echo, formatProcess, getPortOwner, getProcessTable, isChildOf, is_child_of, lsof_argv, process_table, process_table_ts, ps_argv, refresh_process_table, refresh_process_table_if_needed, start, verbose, visitProcessTree, waitForPortOwner, warn;

  ({$, echo, warn, verbose} = require('../common'));

  ChildProcess = require("child_process");

  // need to support some queries
  // Wait until a PID (or a child of PID) owns port N
  ps_argv = ["-eo", "uid,pid,ppid,pcpu,rss,command"];

  lsof_argv = ["-Pni"];

  // Process table entry:
  // 0: { pid: 0, ppid: 0, ports: [], pcpu: 0, rss: 0, command: 0 }
  process_table_ts = 0;

  process_table = {};

  refresh_process_table_if_needed = function(cb) {
    if (Date.now() - process_table_ts > 3000) {
      return refresh_process_table(cb);
    } else {
      return cb(null, process_table);
    }
  };

  refresh_process_table = function(cb) {
    var buf, ps;
    process_table = {};
    ps = ChildProcess.spawn('ps', ps_argv, {
      shell: false
    });
    buf = "";
    ps.stdout.on('data', function(data) {
      return buf += data.toString('utf8');
    });
    ps.on('error', cb);
    ps.on('close', function() {
      var _, command, i, len, line, lsof, lsof_buf, pcpu, pid, ppid, ref, rss, uid;
      ref = buf.split(/\n/);
      for (i = 0, len = ref.length; i < len; i++) {
        line = ref[i];
        [_, uid, pid, ppid, pcpu, rss, ...command] = line.split(/ +/);
        command = command.join(' ');
        if ((pid != null ? pid.length : void 0) > 0 && pid !== "PID") {
          uid = parseInt(uid);
          pid = parseInt(pid);
          ppid = parseInt(ppid);
          pcpu = parseFloat(pcpu);
          rss = parseInt(rss);
          process_table[pid] = {
            uid,
            pid,
            ppid,
            pcpu,
            rss,
            command,
            ports: []
          };
        }
      }
      lsof = ChildProcess.spawn('lsof', lsof_argv, {
        shell: false
      });
      lsof_buf = "";
      lsof.stdout.on('data', function(data) {
        return lsof_buf += data.toString('utf8');
      });
      lsof.on('error', cb);
      return lsof.on('close', function() {
        var addr, dev, fd, j, len1, mode, name, proto, ref1, sz, type, user;
        ref1 = lsof_buf.split(/\n/);
        for (j = 0, len1 = ref1.length; j < len1; j++) {
          line = ref1[j];
          [name, pid, user, fd, type, dev, sz, proto, addr, mode] = line.split(/ +/);
          if (mode === "(LISTEN)") {
            if (!(pid in process_table)) {
              process_table[pid] = {
                pid,
                ppid: 0,
                ports: [],
                pcpu: 0,
                rss: 0,
                command: "???"
              };
            }
            process_table[pid].ports.push(addr);
          }
        }
        process_table_ts = Date.now();
        return cb(null, process_table);
      });
    });
    return null;
  };

  isChildOf = function(ppid, pid, cb) {
    return refresh_process_table_if_needed(function(err, procs) {
      if (err) {
        return cb(err);
      }
      return cb(null, is_child_of(procs, ppid, pid));
    });
  };

  is_child_of = function(procs, ppid, pid) {
    var proc, ref;
    proc = procs[pid];
    if (ppid === pid || ppid === proc.ppid) {
      return true;
    }
    if ((ref = proc.ppid) === proc.pid || ref === 0 || ref === 1 || ref === null || ref === (void 0) || ref === '' || ref === '0' || ref === '1') {
      return false;
    }
    return is_child_of(procs, ppid, proc.ppid);
  };

  visitProcessTree = (pid, visit) => {
    return refresh_process_table_if_needed((err, procs) => {
      var walk;
      procs[pid] && visit(procs[pid], 0);
      walk = (_pid, level) => {
        var _, proc, results;
        results = [];
        for (_ in procs) {
          proc = procs[_];
          if (!((proc != null ? proc.ppid : void 0) === _pid)) {
            continue;
          }
          visit(proc, level);
          results.push(walk(proc.pid, level + 1));
        }
        return results;
      };
      return walk(pid, 1);
    });
  };

  formatProcess = (proc) => {
    return [proc.pid, '[', proc.pcpu.toFixed(1) + "%", $.commaize(Math.round(proc.rss / 1024)) + "MB", ']', $.stringTruncate(proc.command, 100, '...', '/'), proc.ports.length && ('[ ' + proc.ports.join(', ') + ' ]') || ""].join(' ');
  };

  getPortOwner = (port, cb) => {
    port = String(port);
    return refresh_process_table_if_needed((err, procs) => {
      var _, _port, i, len, proc, ref, this_port;
      if (err) {
        return cb(err);
      }
      for (_ in procs) {
        proc = procs[_];
        ref = proc.ports;
        for (i = 0, len = ref.length; i < len; i++) {
          _port = ref[i];
          this_port = _port.split(':')[1];
          if (this_port === port) {
            return cb(null, proc);
          }
        }
      }
      return cb(null, null);
    });
  };

  waitForPortOwner = (target, port, timeout, cb) => {
    var _timeout, checkAgain, done, exit_handler, target_port;
    target_port = String(port);
    _timeout = setTimeout((() => {
      return done('timeout');
    }), timeout);
    target.on('exit', exit_handler = (() => {
      return done('exit');
    }));
    done = (err, proc) => {
      clearTimeout(_timeout);
      target.removeListener('exit', exit_handler);
      if (typeof cb === "function") {
        cb(err, proc);
      }
      return cb = null;
    };
    (checkAgain = () => {
      refresh_process_table((err, procs) => {
        var _, _port, i, len, proc, ref, this_port;
        if (err) {
          return done(err);
        }
        for (_ in procs) {
          proc = procs[_];
          ref = proc.ports;
          for (i = 0, len = ref.length; i < len; i++) {
            _port = ref[i];
            this_port = _port.split(':')[1];
            if (this_port === target_port && is_child_of(procs, target.pid, proc.pid)) {
              return done(null, proc);
            }
          }
        }
        (cb != null) && setTimeout(checkAgain, 400);
        return null;
      });
      return null;
    })();
    return null;
  };

  getProcessTable = refresh_process_table;

  Object.assign(module.exports, {formatProcess, waitForPortOwner, visitProcessTree, getPortOwner, isChildOf, getProcessTable});

  if (require.main === module) {
    start = Date.now();
    refresh_process_table(function(err, procs) {
      var elapsed;
      $.log((elapsed = Date.now() - start) + "ms");
      return waitForPortOwner(854, 27017, 1000, function(err, owner) {
        var cpuSum, memSum;
        if (err === 'timeout') {
          return console.log("TIMED OUT");
        } else {
          console.log(formatProcess(owner));
          visitProcessTree(854, function(proc, level) {
            return console.log('+', $.repeat(". ", level) + formatProcess(proc));
          });
          cpuSum = memSum = 0;
          visitProcessTree(854, function(proc, level) {
            cpuSum += proc.pcpu;
            return memSum += proc.rss;
          });
          return console.log(`CPU: ${cpuSum.toFixed(0)}% MEM: ${$.commaize(Math.round(memSum / 1024))}MB`);
        }
      });
    });
  }

}).call(this);
